/*
 * Sakreble! - open word game.
 * Copyright (C) 2004 Marcin Gajda
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//#include <qapp.h>
#include <stdio.h>

#include <qpainter.h>
#include <qwindefs.h>
#include <qfont.h>

#include "gameboard.h"
#include "dupa.h"
#include "alphabet.h"

Gameboard::Gameboard( QWidget *parent , const char *name ) : QWidget( parent, name )
{
	int i, j;

	for( i=0; i<15; i++ )
		for( j=0; j<15; j++ )
		{
			fields[i][j].letter = EMPTY_FIELD;
			fields[i][j].bonus = NORMAL;
			fields[i][j].added = 0;
		}

	fields[1][1].bonus = TWO_WORD_BONUS;
	fields[2][2].bonus = TWO_WORD_BONUS;
	fields[3][3].bonus = TWO_WORD_BONUS;
	fields[4][4].bonus = TWO_WORD_BONUS;
	fields[7][7].bonus = TWO_WORD_BONUS;
	fields[10][10].bonus = TWO_WORD_BONUS;
	fields[11][11].bonus = TWO_WORD_BONUS;
	fields[12][12].bonus = TWO_WORD_BONUS;
	fields[13][13].bonus = TWO_WORD_BONUS;
	fields[1][13].bonus = TWO_WORD_BONUS;
	fields[2][12].bonus = TWO_WORD_BONUS;
	fields[3][11].bonus = TWO_WORD_BONUS;
	fields[4][10].bonus = TWO_WORD_BONUS;
	fields[10][4].bonus = TWO_WORD_BONUS;
	fields[11][3].bonus = TWO_WORD_BONUS;
	fields[12][2].bonus = TWO_WORD_BONUS;
	fields[13][1].bonus = TWO_WORD_BONUS;
	fields[3][0].bonus = TWO_LETTER_BONUS;
	fields[11][0].bonus = TWO_LETTER_BONUS;
	fields[6][2].bonus = TWO_LETTER_BONUS;
	fields[8][2].bonus = TWO_LETTER_BONUS;
	fields[7][3].bonus = TWO_LETTER_BONUS;
	fields[0][3].bonus = TWO_LETTER_BONUS;
	fields[14][3].bonus = TWO_LETTER_BONUS;
	fields[2][6].bonus = TWO_LETTER_BONUS;
	fields[6][6].bonus = TWO_LETTER_BONUS;
	fields[8][6].bonus = TWO_LETTER_BONUS;
	fields[12][6].bonus = TWO_LETTER_BONUS;
	fields[3][7].bonus = TWO_LETTER_BONUS;
	fields[11][7].bonus = TWO_LETTER_BONUS;
	fields[2][8].bonus = TWO_LETTER_BONUS;
	fields[6][8].bonus = TWO_LETTER_BONUS;
	fields[8][8].bonus = TWO_LETTER_BONUS;
	fields[12][8].bonus = TWO_LETTER_BONUS;
	fields[0][11].bonus = TWO_LETTER_BONUS;
	fields[7][11].bonus = TWO_LETTER_BONUS;
	fields[14][11].bonus = TWO_LETTER_BONUS;
	fields[6][12].bonus = TWO_LETTER_BONUS;
	fields[8][12].bonus = TWO_LETTER_BONUS;
	fields[3][14].bonus = TWO_LETTER_BONUS;
	fields[11][14].bonus = TWO_LETTER_BONUS;
	fields[5][1].bonus = THREE_LETTER_BONUS;
	fields[9][1].bonus = THREE_LETTER_BONUS;
	fields[1][5].bonus = THREE_LETTER_BONUS;
	fields[5][5].bonus = THREE_LETTER_BONUS;
	fields[9][5].bonus = THREE_LETTER_BONUS;
	fields[13][5].bonus = THREE_LETTER_BONUS;
	fields[1][9].bonus = THREE_LETTER_BONUS;
	fields[5][9].bonus = THREE_LETTER_BONUS;
	fields[9][9].bonus = THREE_LETTER_BONUS;
	fields[13][9].bonus = THREE_LETTER_BONUS;
	fields[5][13].bonus = THREE_LETTER_BONUS;
	fields[9][13].bonus = THREE_LETTER_BONUS;
	fields[0][0].bonus = THREE_WORD_BONUS;
	fields[7][0].bonus = THREE_WORD_BONUS;
	fields[14][0].bonus = THREE_WORD_BONUS;
	fields[0][7].bonus = THREE_WORD_BONUS;
	fields[14][7].bonus = THREE_WORD_BONUS;
	fields[0][14].bonus = THREE_WORD_BONUS;
	fields[7][14].bonus = THREE_WORD_BONUS;
	fields[14][14].bonus = THREE_WORD_BONUS;

	highlightingXPos = highlightingYPos = -1;
}

const QRect gameboardRect( 0 , 0 , 450 , 450 );

void Gameboard::paintEvent( QPaintEvent *e )
{
	QPainter paint;
	QString s;
	QRect updateR = e->rect();

	paint.begin( this );

	paint.setPen( black );

	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
		{
			if(	!updateR.intersects( QRect( i*30+1 , j*30+1 , 28 , 28 ) ) )
				continue;

			if( fields[i][j].letter == EMPTY_FIELD )
				switch( fields[i][j].bonus )
				{
					case TWO_LETTER_BONUS:
						paint.setBrush( blue );
						break;
					case THREE_LETTER_BONUS:
						paint.setBrush( darkBlue );
						break;
					case TWO_WORD_BONUS:
						paint.setBrush( red );
						break;
					case THREE_WORD_BONUS:
						paint.setBrush( darkRed );
						break;
					default:
						paint.setBrush( darkGreen );
						break;
				}
			else
			{
				if( highlightingXPos == i && highlightingYPos == j )
					paint.setBrush( yellow );
				else
				{
					if( fields[i][j].added )
						paint.setBrush( lightGray );
					else
						paint.setBrush( darkGray );
				}
			}

			paint.drawRect( i*30 + 1 , j*30 + 1 , 28 , 28 );

			if( fields[i][j].letter != EMPTY_FIELD )
			{
				s.sprintf( "%c" , alphabet->upperCaseDependOnPolishLettersVisibility( fields[i][j].letter ) );
				paint.setFont( QFont( "Times" , 18 , QFont::Bold ) );
				paint.drawText( i*30 + 1 , j*30 + 1 , 26 , 26 , AlignCenter , s , 1 );

				if( alphabet->getPolishLettersVisibility() && alphabet->isOgonekNeeded( fields[i][j].letter ) )
				{
					s = tr( "%1" ).arg( alphabet->addOgonek( fields[i][j].letter ) );

					int vert_offset = alphabet->whereVerticalOgonek( fields[i][j].letter );
					int hor_offset  = alphabet->whereHorizontalOgonek( fields[i][j].letter );

					paint.drawText( i*30 - 1 + hor_offset , j*30 - 1 + vert_offset , s , 1 );
				}

				if( alphabet->getLetterPoints( fields[i][j].letter ) > 0 )
				{
					s.sprintf( "%i" , alphabet->getLetterPoints( fields[i][j].letter ) );
					paint.setFont( QFont( "Times" , 10 , QFont::Bold ) );
					paint.drawText( i*30 + 19 , j*30 + 16 , 10 , 10 , AlignCenter , s , 1 );
				}
			}
			else
			{
				if( fields[i][j].bonus != NORMAL )
				{
					paint.setFont( QFont( "Arial" , 10 , QFont::Bold ) );
					switch( fields[i][j].bonus )
					{
						case TWO_LETTER_BONUS:
							s.sprintf( "2L" );
							paint.drawText( i*30+1 , j*30+1 , 28 , 28 , AlignCenter , s , 2 );
							break;
						case THREE_LETTER_BONUS:
							s.sprintf( "3L" );
							paint.drawText( i*30+1 , j*30+1 , 28 , 28 , AlignCenter , s , 2 );
							break;
						case TWO_WORD_BONUS:
							s.sprintf( "2W" );
							paint.drawText( i*30+1 , j*30+1 , 28 , 28 , AlignCenter , s , 2 );
							break;
						case THREE_WORD_BONUS:
							s.sprintf( "3W" );
							paint.drawText( i*30+1 , j*30+1 , 28 , 28 , AlignCenter , s , 2 );
							break;
						default:;
					}
				}
			}
		}

	paint.end();
}

void Gameboard::mousePressEvent( QMouseEvent *e )
{
	if ( e->button() != LeftButton )
		return;

	emit gameboardClicked( e->x() / 30 , e->y() / 30 );
}

/*
 * Tu rzeczy nie zwiazane z rysowaniem
 */

int Gameboard::bonus( int i , int j )
{
	return fields[i][j].bonus;
}

char Gameboard::letter( int i , int j )
{
	return fields[i][j].letter;
}

void Gameboard::setLetter( char l , int i , int j )
{
	if( fields[i][j].letter == l )
		return;

	fields[i][j].letter = l;
	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
}

void Gameboard::setBonus( int p , int i , int j )
{
	fields[i][j].bonus = p;
}

int Gameboard::isWordCorrectlyAdded()
{
	int firstAddedLetterXPos = -1, firstAddedLetterYPos = -1;
	int found_not_added_letter = 0;
	int numberOfAddedLetters = 0, number_of_added_letters = 0;
	int horizont;

	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].added )
			{
				if( firstAddedLetterXPos == -1 )
				{
					firstAddedLetterXPos = i;
					firstAddedLetterYPos = j;
				}

				numberOfAddedLetters++;
			}

	horizont = VERTICAL;

	for( int i=firstAddedLetterXPos+1; i<15; i++ )
	{
		if( fields[i][firstAddedLetterYPos].added )
			horizont = HORIZONTAL;
	}

	while( firstAddedLetterXPos < 15 && firstAddedLetterYPos < 15 )
	{
		if( fields[firstAddedLetterXPos][firstAddedLetterYPos].letter == EMPTY_FIELD )
			break;

		if( firstAddedLetterXPos > 0 )
			if( !fields[firstAddedLetterXPos-1][firstAddedLetterYPos].added && fields[firstAddedLetterXPos-1][firstAddedLetterYPos].letter != EMPTY_FIELD )
				found_not_added_letter = 1;

		if( firstAddedLetterYPos > 0 )
			if( !fields[firstAddedLetterXPos][firstAddedLetterYPos-1].added && fields[firstAddedLetterXPos][firstAddedLetterYPos-1].letter != EMPTY_FIELD )
				found_not_added_letter = 1;

		if( firstAddedLetterXPos < 14 )
			if( !fields[firstAddedLetterXPos+1][firstAddedLetterYPos].added && fields[firstAddedLetterXPos+1][firstAddedLetterYPos].letter != EMPTY_FIELD )
				found_not_added_letter = 1;

		if( firstAddedLetterYPos < 14 )
			if( !fields[firstAddedLetterXPos][firstAddedLetterYPos+1].added && fields[firstAddedLetterXPos][firstAddedLetterYPos+1].letter != EMPTY_FIELD )
				found_not_added_letter = 1;

		if( fields[firstAddedLetterXPos][firstAddedLetterYPos].added )
			number_of_added_letters++;

		if( horizont == HORIZONTAL )
			firstAddedLetterXPos++;
		else
			firstAddedLetterYPos++;
	}

	if( number_of_added_letters == numberOfAddedLetters )
	{
		if( found_not_added_letter )
			return 1;
		else
			if( fields[7][7].added )
				return 1;
	}

	return 0;
}

void Gameboard::highlightLetterUstawionaPrzezKomputer( char l , int i , int j )
{
	fields[i][j].letter = l;
	fields[i][j].added = 1;

	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
}

void Gameboard::setAddedLetter( char l , int i , int j )
{
	fields[i][j].letter = l;
	fields[i][j].added = 1;

	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );

	if( isWordCorrectlyAdded() )
		emit enableOkButton( true );
	else
		emit enableOkButton( false );
}

void Gameboard::removeAddedLetter( int i , int j )
{
	if( !fields[i][j].added )
		return;

	fields[i][j].letter = EMPTY_FIELD;
	fields[i][j].added = 0;

	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );

	if( isWordCorrectlyAdded() )
		emit enableOkButton( true );
	else
		emit enableOkButton( false );
}

void Gameboard::removeAddedLetters()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].added )
				removeAddedLetter( i , j );
}

void Gameboard::highlightLetter( int i , int j )
{
	if( fields[i][j].letter == EMPTY_FIELD )
		return;

	if( i == highlightingXPos && j == highlightingYPos )
		return;

	if( !fields[i][j].added )
		return;

	highlightingXPos = i;
	highlightingYPos = j;

	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
}

void Gameboard::turnOffLetter()
{
	int i, j;

	i = highlightingXPos;
	j = highlightingYPos;

	highlightingXPos = -1;
	highlightingYPos = -1;

	repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
}

int Gameboard::thereAreAddedLetters()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].added )
				return 1;

	return 0;
}

int Gameboard::thereAreLetters()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].letter != EMPTY_FIELD )
				return 1;

	return 0;
}

int Gameboard::getPointValue( int i , int j , int horizont )
{
	int points = 0;
	int number_of_added_letters = 0;
	int word_multiplikator = 1;

	while( i<15 && j<15 )
	{
		if( fields[i][j].letter == EMPTY_FIELD )
			break;

		if( !fields[i][j].added )
			points += alphabet->getLetterPoints( fields[i][j].letter );
		else
		{
			number_of_added_letters++;

			switch( fields[i][j].bonus )
			{
				case TWO_LETTER_BONUS:
					points += 2 * alphabet->getLetterPoints( fields[i][j].letter );
					break;

				case THREE_LETTER_BONUS:
					points += 3 * alphabet->getLetterPoints( fields[i][j].letter );
					break;

				case TWO_WORD_BONUS:
					points += alphabet->getLetterPoints( fields[i][j].letter );
					word_multiplikator *= 2;
					break;

				case THREE_WORD_BONUS:
					points += alphabet->getLetterPoints( fields[i][j].letter );
					word_multiplikator *= 3;
					break;

				default:
					points += alphabet->getLetterPoints( fields[i][j].letter );
					break;
			}
		}

		if( horizont == HORIZONTAL )
			i++;
		else
			j++;
	}

	points *= word_multiplikator;

	if( number_of_added_letters == 7 )
		points += 50;

	return points;
}

int Gameboard::getPointValue()
{
	int firstAddedLetterXPos = -1, firstAddedLetterYPos = -1, direction_up = 0, direction_left = 0;
	int horizont = HORIZONTAL;
	int points = 0, second_letter_added = 0;

	if( isWordCorrectlyAdded() )
	{
		for( int i=0; i<15; i++ )
			for( int j=0; j<15; j++ )
				if( fields[i][j].added )
				{
					if( firstAddedLetterXPos == -1 )
					{
						firstAddedLetterXPos = i;
						firstAddedLetterYPos = j;
					}
					break;
				}

		for( int i=firstAddedLetterXPos+1; i<15; i++ )
			if( addedLetter( i , firstAddedLetterYPos ) )
			{
				horizont = HORIZONTAL;
				second_letter_added = 1;
				break;
			}

		if( !second_letter_added )
			for( int j=firstAddedLetterYPos+1; j<15; j++ )
				if( addedLetter( firstAddedLetterXPos , j ) )
				{
					horizont = VERTICAL;
					second_letter_added = 1;
					break;
				}

		if( !second_letter_added )
		{
			if( firstAddedLetterXPos > 0 && letter( firstAddedLetterXPos - 1 , firstAddedLetterYPos ) != EMPTY_FIELD )
				horizont = HORIZONTAL;

			if( firstAddedLetterXPos < 14 && letter( firstAddedLetterXPos + 1 , firstAddedLetterYPos ) != EMPTY_FIELD )
				horizont = HORIZONTAL;

			if( firstAddedLetterYPos > 0 && letter( firstAddedLetterXPos , firstAddedLetterYPos - 1 ) != EMPTY_FIELD )
				horizont = VERTICAL;

			if( firstAddedLetterYPos < 14 && letter( firstAddedLetterXPos , firstAddedLetterYPos + 1 ) != EMPTY_FIELD )
				horizont = VERTICAL;
		}

		if( horizont == HORIZONTAL )
			while( firstAddedLetterXPos > 0 && fields[firstAddedLetterXPos-1][firstAddedLetterYPos].letter != EMPTY_FIELD )
				firstAddedLetterXPos--;
		else
			while( firstAddedLetterYPos > 0 && fields[firstAddedLetterXPos][firstAddedLetterYPos-1].letter != EMPTY_FIELD )
				firstAddedLetterYPos--;

		points += getPointValue( firstAddedLetterXPos , firstAddedLetterYPos , horizont );

		while( firstAddedLetterXPos < 15 && firstAddedLetterYPos < 15 )
		{
			if( fields[ firstAddedLetterXPos ][ firstAddedLetterYPos ].letter == EMPTY_FIELD )
				break;

			if( addedLetter( firstAddedLetterXPos , firstAddedLetterYPos ) )
			{
				if( horizont == HORIZONTAL )
				{
					direction_up = 0;
					while( firstAddedLetterYPos-direction_up-1 >= 0 )
						if( letter( firstAddedLetterXPos , firstAddedLetterYPos-direction_up-1 ) != EMPTY_FIELD )
							direction_up++;
						else
							break;

					if( direction_up > 0 || ( firstAddedLetterYPos < 14 && letter( firstAddedLetterXPos , firstAddedLetterYPos + 1 ) != EMPTY_FIELD ) )
						points += getPointValue( firstAddedLetterXPos , firstAddedLetterYPos-direction_up , VERTICAL );
				}
				else
				{
					direction_left = 0;
					while( firstAddedLetterXPos-direction_left-1 >= 0 )
						if( letter( firstAddedLetterXPos-direction_left-1 , firstAddedLetterYPos ) != EMPTY_FIELD )
							direction_left++;
						else
							break;

					if( direction_left > 0 || ( firstAddedLetterXPos < 14 && letter( firstAddedLetterXPos + 1 , firstAddedLetterYPos ) != EMPTY_FIELD ) )
						points += getPointValue( firstAddedLetterXPos-direction_left , firstAddedLetterYPos , HORIZONTAL );
				}
			}

			if( horizont == HORIZONTAL )
				firstAddedLetterXPos++;
			else
				firstAddedLetterYPos++;
		}
	}

	return points;
}

void Gameboard::removeAddingIndicatorFromAddedLetters()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].added )
			{
				fields[i][j].added = 0;
				repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
			}	
}

void Gameboard::clearGameboard()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
		{
			fields[i][j].letter = EMPTY_FIELD;
			fields[i][j].added = 0;
		}

	repaint( QRect( 0 , 0 , 450 , 450 ) );
}

int Gameboard::isBlankAdded()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].letter == BLANK )
				return 1;

	return 0;
}

void Gameboard::changeBlank( char to_what )
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].letter == BLANK )
			{
				fields[i][j].letter = alphabet->upperCase( to_what );
				repaint( QRect( i*30+1 , j*30+1 , 28 , 28 ) );
				return;
			}

}

void Gameboard::highlightBlank()
{
	for( int i=0; i<15; i++ )
		for( int j=0; j<15; j++ )
			if( fields[i][j].letter == BLANK )
			{
				highlightLetter( i , j );
				return;
			}

}

void Gameboard::turnOffBlank()
{
	turnOffLetter();
}
